###
### This file was automatically generated
###

from archinfo.arch import register_arch, Endness, Register

from .common import ArchPcode


class ArchPcode_SuperH_BE_32_SH_2(ArchPcode):
    name = 'SuperH:BE:32:SH-2'
    pcode_arch = 'SuperH:BE:32:SH-2'
    description = 'SuperH SH-2 processor 32-bit big-endian'
    bits = 32
    ip_offset = 0x118
    sp_offset = 0x3c
    bp_offset = sp_offset
    instruction_endness = Endness.BE
    register_list = [
        Register('r0', 4, 0x0),
        Register('r1', 4, 0x4),
        Register('r2', 4, 0x8),
        Register('r3', 4, 0xc),
        Register('r4', 4, 0x10),
        Register('r5', 4, 0x14),
        Register('r6', 4, 0x18),
        Register('r7', 4, 0x1c),
        Register('r8', 4, 0x20),
        Register('r9', 4, 0x24),
        Register('r10', 4, 0x28),
        Register('r11', 4, 0x2c),
        Register('r12', 4, 0x30),
        Register('r13', 4, 0x34),
        Register('r14', 4, 0x38),
        Register('r15', 4, 0x3c),
        Register('sr', 4, 0x100),
        Register('gbr', 4, 0x104),
        Register('vbr', 4, 0x108),
        Register('mach', 4, 0x10c),
        Register('macl', 4, 0x110),
        Register('pr', 4, 0x114),
        Register('pc', 4, 0x118, alias_names=('ip',))
    ]

register_arch(['superh:be:32:sh-2'], 32, Endness.BE, ArchPcode_SuperH_BE_32_SH_2)




import random

from ..func import Func, TestData


def rand_str(length, byte_list=None):
    if byte_list is None:
        return "".join(chr(random.randint(0, 255)) for _ in range(length))
    return "".join(random.choice(byte_list) for _ in range(length))


class memcmp(Func):
    def __init__(self):
        super(memcmp, self).__init__() #pylint disable=useless-super-delegation

    def get_name(self):
        return "memcmp"

    def num_args(self):
        return 3

    def args(self): #pylint disable=no-self-use
        return ["buf1", "buf2", "len"]

    def gen_input_output_pair(self):
        return None

    def can_call_other_funcs(self):
        return False

    def pre_test(self, func, runner):
        # todo we don't test which order it returns the signs in
        l = random.randint(1, 20)
        bufb = rand_str(l)
        bufa = bufb + rand_str(5)
        test_input = [bufa, bufb, l]
        test_output = [bufa, bufb, l]
        max_steps = 10
        return_val = None
        test = TestData(test_input, test_output, return_val, max_steps)
        s = runner.get_out_state(func, test)
        if s is None or s.solver.eval(s.regs.eax) != 0:
            return False

        bufa = "asd\x00a"
        bufb = "asd\x00a"
        test_input = [bufa, bufb, 5]
        test_output = [bufa, bufb, 5]
        test = TestData(test_input, test_output, return_val, max_steps)
        x = runner.get_out_state(func, test)
        if x is None:
            return False
        outval1 = x.solver.eval(x.regs.eax)

        bufa = "asd\x00c"
        bufb = "asd\x00b"
        test_input = [bufa, bufb, 5]
        test_output = [bufa, bufb, 5]
        test = TestData(test_input, test_output, return_val, max_steps)
        s = runner.get_out_state(func, test)
        if s is None:
            return False

        outval2 = s.solver.eval(s.regs.eax)

        if outval1 != 0 or outval2 == 0:
            return False


        return True



import logging
from typing import Iterable, Tuple, Any

from .. import MemoryMixin

l = logging.getLogger(name=__name__)


class AbstractMergerMixin(MemoryMixin):

    def _merge_values(self, values: Iterable[Tuple[Any,Any]], merged_size: int, **kwargs):

        # if self.category == 'reg' and self.state.arch.register_endness == 'Iend_LE':
        #     should_reverse = True
        # elif self.state.arch.memory_endness == 'Iend_LE':
        #     should_reverse = True
        # else:
        #     should_reverse = False

        values = list(values)
        merged_val = values[0][0]

        # if should_reverse: merged_val = merged_val.reversed

        for tm, _ in values[1:]:
            # if should_reverse: tm = tm.reversed

            if self._is_uninitialized(tm):
                continue
            l.info("Merging %s %s...", merged_val, tm)
            merged_val = merged_val.union(tm)
            l.info("... Merged to %s", merged_val)

        # if should_reverse:
        #     merged_val = merged_val.reversed

        if not values[0][0].uninitialized and self.state.solver.backends.vsa.identical(merged_val, values[0][0]):
            return None

        return merged_val

    @staticmethod
    def _is_uninitialized(a):
        return getattr(a._model_vsa, 'uninitialized', False)



###
### This file was automatically generated
###

from archinfo.arch import register_arch, Endness, Register

from .common import ArchPcode


class ArchPcode_z8401x_LE_16_default(ArchPcode):
    name = 'z8401x:LE:16:default'
    pcode_arch = 'z8401x:LE:16:default'
    description = 'Zilog Z8401x (IPC) microcontroller'
    bits = 16
    ip_offset = 0x42
    sp_offset = 0x44
    bp_offset = sp_offset
    instruction_endness = Endness.LE
    register_list = [
        Register('af', 2, 0x0),
        Register('f', 1, 0x0),
        Register('a', 1, 0x1),
        Register('bc', 2, 0x2),
        Register('c', 1, 0x2),
        Register('b', 1, 0x3),
        Register('de', 2, 0x4),
        Register('e', 1, 0x4),
        Register('d', 1, 0x5),
        Register('hl', 2, 0x6),
        Register('l', 1, 0x6),
        Register('h', 1, 0x7),
        Register('i', 1, 0x8),
        Register('r', 1, 0x9),
        Register('af_', 2, 0x20),
        Register('f_', 1, 0x20),
        Register('a_', 1, 0x21),
        Register('bc_', 2, 0x22),
        Register('c_', 1, 0x22),
        Register('b_', 1, 0x23),
        Register('de_', 2, 0x24),
        Register('e_', 1, 0x24),
        Register('d_', 1, 0x25),
        Register('hl_', 2, 0x26),
        Register('l_', 1, 0x26),
        Register('h_', 1, 0x27),
        Register('pc', 2, 0x42, alias_names=('ip',)),
        Register('sp', 2, 0x44),
        Register('ix', 2, 0x46),
        Register('iy', 2, 0x48),
        Register('rcbar', 1, 0x50),
        Register('rcbr', 1, 0x51),
        Register('rbbr', 1, 0x52),
        Register('decompile_mode', 1, 0x60),
        Register('contextreg', 4, 0xf0)
    ]

register_arch(['z8401x:le:16:default'], 16, Endness.LE, ArchPcode_z8401x_LE_16_default)



# pylint:disable=line-too-long
import logging

from ...sim_type import SimTypeFunction,     SimTypeShort, SimTypeInt, SimTypeLong, SimTypeLongLong, SimTypeDouble, SimTypeFloat,     SimTypePointer,     SimTypeChar,     SimStruct,     SimTypeFixedSizeArray,     SimTypeBottom,     SimUnion,     SimTypeBool
from ...calling_conventions import SimCCStdcall, SimCCMicrosoftAMD64
from .. import SIM_PROCEDURES as P
from . import SimLibrary


_l = logging.getLogger(name=__name__)


lib = SimLibrary()
lib.set_default_cc('X86', SimCCStdcall)
lib.set_default_cc('AMD64', SimCCMicrosoftAMD64)
lib.set_library_names("cabinet.dll")
prototypes = \
    {
        # 
        'FCICreate': SimTypeFunction([SimTypePointer(SimStruct({"erfOper": SimTypeInt(signed=True, label="Int32"), "erfType": SimTypeInt(signed=True, label="Int32"), "fError": SimTypeInt(signed=True, label="Int32")}, name="ERF", pack=False, align=None), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimStruct({"cb": SimTypeInt(signed=False, label="UInt32"), "cbFolderThresh": SimTypeInt(signed=False, label="UInt32"), "cbReserveCFHeader": SimTypeInt(signed=False, label="UInt32"), "cbReserveCFFolder": SimTypeInt(signed=False, label="UInt32"), "cbReserveCFData": SimTypeInt(signed=False, label="UInt32"), "iCab": SimTypeInt(signed=True, label="Int32"), "iDisk": SimTypeInt(signed=True, label="Int32"), "fFailOnIncompressible": SimTypeInt(signed=True, label="Int32"), "setID": SimTypeShort(signed=False, label="UInt16"), "szDisk": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 256), "szCab": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 256), "szCabPath": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 256)}, name="CCAB", pack=False, align=None), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pccab", "pszFile", "cbFile", "fContinuation", "pv"]), offset=0), SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["cb"]), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["memory"]), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["pszFile", "oflag", "pmode", "err", "pv"]), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hf", "memory", "cb", "err", "pv"]), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["hf", "memory", "cb", "err", "pv"]), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hf", "err", "pv"]), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hf", "dist", "seektype", "err", "pv"]), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pszFile", "err", "pv"]), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pszTempName", "cbTempName", "pv"]), offset=0), SimTypePointer(SimStruct({"cb": SimTypeInt(signed=False, label="UInt32"), "cbFolderThresh": SimTypeInt(signed=False, label="UInt32"), "cbReserveCFHeader": SimTypeInt(signed=False, label="UInt32"), "cbReserveCFFolder": SimTypeInt(signed=False, label="UInt32"), "cbReserveCFData": SimTypeInt(signed=False, label="UInt32"), "iCab": SimTypeInt(signed=True, label="Int32"), "iDisk": SimTypeInt(signed=True, label="Int32"), "fFailOnIncompressible": SimTypeInt(signed=True, label="Int32"), "setID": SimTypeShort(signed=False, label="UInt16"), "szDisk": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 256), "szCab": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 256), "szCabPath": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 256)}, name="CCAB", pack=False, align=None), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["perf", "pfnfcifp", "pfna", "pfnf", "pfnopen", "pfnread", "pfnwrite", "pfnclose", "pfnseek", "pfndelete", "pfnfcigtf", "pccab", "pv"]),
        # 
        'FCIAddFile': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeFunction([SimTypePointer(SimStruct({"cb": SimTypeInt(signed=False, label="UInt32"), "cbFolderThresh": SimTypeInt(signed=False, label="UInt32"), "cbReserveCFHeader": SimTypeInt(signed=False, label="UInt32"), "cbReserveCFFolder": SimTypeInt(signed=False, label="UInt32"), "cbReserveCFData": SimTypeInt(signed=False, label="UInt32"), "iCab": SimTypeInt(signed=True, label="Int32"), "iDisk": SimTypeInt(signed=True, label="Int32"), "fFailOnIncompressible": SimTypeInt(signed=True, label="Int32"), "setID": SimTypeShort(signed=False, label="UInt16"), "szDisk": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 256), "szCab": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 256), "szCabPath": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 256)}, name="CCAB", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pccab", "cbPrevCab", "pv"]), offset=0), SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["typeStatus", "cb1", "cb2", "pv"]), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["pszName", "pdate", "ptime", "pattribs", "err", "pv"]), offset=0), SimTypeShort(signed=False, label="UInt16")], SimTypeInt(signed=True, label="Int32"), arg_names=["hfci", "pszSourceFile", "pszFileName", "fExecute", "pfnfcignc", "pfnfcis", "pfnfcigoi", "typeCompress"]),
        # 
        'FCIFlushCabinet': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeFunction([SimTypePointer(SimStruct({"cb": SimTypeInt(signed=False, label="UInt32"), "cbFolderThresh": SimTypeInt(signed=False, label="UInt32"), "cbReserveCFHeader": SimTypeInt(signed=False, label="UInt32"), "cbReserveCFFolder": SimTypeInt(signed=False, label="UInt32"), "cbReserveCFData": SimTypeInt(signed=False, label="UInt32"), "iCab": SimTypeInt(signed=True, label="Int32"), "iDisk": SimTypeInt(signed=True, label="Int32"), "fFailOnIncompressible": SimTypeInt(signed=True, label="Int32"), "setID": SimTypeShort(signed=False, label="UInt16"), "szDisk": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 256), "szCab": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 256), "szCabPath": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 256)}, name="CCAB", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pccab", "cbPrevCab", "pv"]), offset=0), SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["typeStatus", "cb1", "cb2", "pv"]), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hfci", "fGetNextCab", "pfnfcignc", "pfnfcis"]),
        # 
        'FCIFlushFolder': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimStruct({"cb": SimTypeInt(signed=False, label="UInt32"), "cbFolderThresh": SimTypeInt(signed=False, label="UInt32"), "cbReserveCFHeader": SimTypeInt(signed=False, label="UInt32"), "cbReserveCFFolder": SimTypeInt(signed=False, label="UInt32"), "cbReserveCFData": SimTypeInt(signed=False, label="UInt32"), "iCab": SimTypeInt(signed=True, label="Int32"), "iDisk": SimTypeInt(signed=True, label="Int32"), "fFailOnIncompressible": SimTypeInt(signed=True, label="Int32"), "setID": SimTypeShort(signed=False, label="UInt16"), "szDisk": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 256), "szCab": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 256), "szCabPath": SimTypeFixedSizeArray(SimTypeBottom(label="CHAR"), 256)}, name="CCAB", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pccab", "cbPrevCab", "pv"]), offset=0), SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["typeStatus", "cb1", "cb2", "pv"]), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hfci", "pfnfcignc", "pfnfcis"]),
        # 
        'FCIDestroy': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hfci"]),
        # 
        'FDICreate': SimTypeFunction([SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32")], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["cb"]), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["pv"]), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32")], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["pszFile", "oflag", "pmode"]), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hf", "pv", "cb"]), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["hf", "pv", "cb"]), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hf"]), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hf", "dist", "seektype"]), offset=0), SimTypeInt(signed=False, label="FDICREATE_CPU_TYPE"), SimTypePointer(SimStruct({"erfOper": SimTypeInt(signed=True, label="Int32"), "erfType": SimTypeInt(signed=True, label="Int32"), "fError": SimTypeInt(signed=True, label="Int32")}, name="ERF", pack=False, align=None), offset=0)], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["pfnalloc", "pfnfree", "pfnopen", "pfnread", "pfnwrite", "pfnclose", "pfnseek", "cpuType", "perf"]),
        # 
        'FDIIsCabinet': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"cbCabinet": SimTypeInt(signed=True, label="Int32"), "cFolders": SimTypeShort(signed=False, label="UInt16"), "cFiles": SimTypeShort(signed=False, label="UInt16"), "setID": SimTypeShort(signed=False, label="UInt16"), "iCabinet": SimTypeShort(signed=False, label="UInt16"), "fReserve": SimTypeInt(signed=True, label="Int32"), "hasprev": SimTypeInt(signed=True, label="Int32"), "hasnext": SimTypeInt(signed=True, label="Int32")}, name="FDICABINETINFO", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hfdi", "hf", "pfdici"]),
        # 
        'FDICopy': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="FDINOTIFICATIONTYPE"), SimTypePointer(SimStruct({"cb": SimTypeInt(signed=True, label="Int32"), "psz1": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "psz2": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "psz3": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "pv": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "hf": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "date": SimTypeShort(signed=False, label="UInt16"), "time": SimTypeShort(signed=False, label="UInt16"), "attribs": SimTypeShort(signed=False, label="UInt16"), "setID": SimTypeShort(signed=False, label="UInt16"), "iCabinet": SimTypeShort(signed=False, label="UInt16"), "iFolder": SimTypeShort(signed=False, label="UInt16"), "fdie": SimTypeInt(signed=False, label="FDIERROR")}, name="FDINOTIFICATION", pack=False, align=None), offset=0)], SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), arg_names=["fdint", "pfdin"]), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimStruct({"fdidt": SimTypeInt(signed=False, label="FDIDECRYPTTYPE"), "pvUser": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "Anonymous": SimUnion({"cabinet": SimStruct({"pHeaderReserve": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "cbHeaderReserve": SimTypeShort(signed=False, label="UInt16"), "setID": SimTypeShort(signed=False, label="UInt16"), "iCabinet": SimTypeInt(signed=True, label="Int32")}, name="_cabinet_e__Struct", pack=False, align=None), "folder": SimStruct({"pFolderReserve": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "cbFolderReserve": SimTypeShort(signed=False, label="UInt16"), "iFolder": SimTypeShort(signed=False, label="UInt16")}, name="_folder_e__Struct", pack=False, align=None), "decrypt": SimStruct({"pDataReserve": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "cbDataReserve": SimTypeShort(signed=False, label="UInt16"), "pbData": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "cbData": SimTypeShort(signed=False, label="UInt16"), "fSplit": SimTypeInt(signed=True, label="Int32"), "cbPartial": SimTypeShort(signed=False, label="UInt16")}, name="_decrypt_e__Struct", pack=False, align=None)}, name="<anon>", label="None")}, name="FDIDECRYPT", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pfdid"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hfdi", "pszCabinet", "pszCabPath", "flags", "pfnfdin", "pfnfdid", "pvUser"]),
        # 
        'FDIDestroy': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hfdi"]),
        # 
        'FDITruncateCabinet': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeShort(signed=False, label="UInt16")], SimTypeInt(signed=True, label="Int32"), arg_names=["hfdi", "pszCabinetName", "iFolderToDelete"]),
        # 
        'CreateCompressor': SimTypeFunction([SimTypeInt(signed=False, label="COMPRESS_ALGORITHM"), SimTypePointer(SimStruct({"Allocate": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["UserContext", "Size"]), offset=0), "Free": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["UserContext", "Memory"]), offset=0), "UserContext": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="COMPRESS_ALLOCATION_ROUTINES", pack=False, align=None), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Algorithm", "AllocationRoutines", "CompressorHandle"]),
        # 
        'SetCompressorInformation': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="COMPRESS_INFORMATION_CLASS"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["CompressorHandle", "CompressInformationClass", "CompressInformation", "CompressInformationSize"]),
        # 
        'QueryCompressorInformation': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="COMPRESS_INFORMATION_CLASS"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["CompressorHandle", "CompressInformationClass", "CompressInformation", "CompressInformationSize"]),
        # 
        'Compress': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["CompressorHandle", "UncompressedData", "UncompressedDataSize", "CompressedBuffer", "CompressedBufferSize", "CompressedDataSize"]),
        # 
        'ResetCompressor': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["CompressorHandle"]),
        # 
        'CloseCompressor': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["CompressorHandle"]),
        # 
        'CreateDecompressor': SimTypeFunction([SimTypeInt(signed=False, label="COMPRESS_ALGORITHM"), SimTypePointer(SimStruct({"Allocate": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["UserContext", "Size"]), offset=0), "Free": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["UserContext", "Memory"]), offset=0), "UserContext": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="COMPRESS_ALLOCATION_ROUTINES", pack=False, align=None), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["Algorithm", "AllocationRoutines", "DecompressorHandle"]),
        # 
        'SetDecompressorInformation': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="COMPRESS_INFORMATION_CLASS"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DecompressorHandle", "CompressInformationClass", "CompressInformation", "CompressInformationSize"]),
        # 
        'QueryDecompressorInformation': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="COMPRESS_INFORMATION_CLASS"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DecompressorHandle", "CompressInformationClass", "CompressInformation", "CompressInformationSize"]),
        # 
        'Decompress': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DecompressorHandle", "CompressedData", "CompressedDataSize", "UncompressedBuffer", "UncompressedBufferSize", "UncompressedDataSize"]),
        # 
        'ResetDecompressor': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DecompressorHandle"]),
        # 
        'CloseDecompressor': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["DecompressorHandle"]),
    }

lib.set_prototypes(prototypes)



###
### This file was automatically generated
###

from archinfo.arch import register_arch, Endness, Register

from .common import ArchPcode


class ArchPcode_CR16C_LE_16_default(ArchPcode):
    name = 'CR16C:LE:16:default'
    pcode_arch = 'CR16C:LE:16:default'
    description = "National Semiconductor's CompactRISC CR16C little endian"
    bits = 16
    ip_offset = 0x32
    sp_offset = 0x24
    bp_offset = sp_offset
    instruction_endness = Endness.LE
    register_list = [
        Register('r1r0', 4, 0x0),
        Register('r0', 2, 0x0),
        Register('r2r1', 4, 0x2),
        Register('r1', 2, 0x2),
        Register('r3r2', 4, 0x4),
        Register('r2', 2, 0x4),
        Register('r4r3', 4, 0x6),
        Register('r3', 2, 0x6),
        Register('r5r4', 4, 0x8),
        Register('r4', 2, 0x8),
        Register('r6r5', 4, 0xa),
        Register('r5', 2, 0xa),
        Register('r7r6', 4, 0xc),
        Register('r6', 2, 0xc),
        Register('r8r7', 4, 0xe),
        Register('r7', 2, 0xe),
        Register('r9r8', 4, 0x10),
        Register('r8', 2, 0x10),
        Register('r10r9', 4, 0x12),
        Register('r9', 2, 0x12),
        Register('r11r10', 4, 0x14),
        Register('r10', 2, 0x14),
        Register('r12lr11', 4, 0x16),
        Register('r11', 2, 0x16),
        Register('r12', 4, 0x18),
        Register('r12_l', 2, 0x18),
        Register('r12_h', 2, 0x1a),
        Register('r13', 4, 0x1c),
        Register('r13_l', 2, 0x1c),
        Register('r13_h', 2, 0x1e),
        Register('ra', 4, 0x20),
        Register('ra_l', 2, 0x20),
        Register('ra_h', 2, 0x22),
        Register('sp', 4, 0x24),
        Register('sp_l', 2, 0x24),
        Register('sp_h', 2, 0x26),
        Register('pc', 4, 0x32, alias_names=('ip',)),
        Register('isp', 4, 0x3c),
        Register('isph', 2, 0x3c),
        Register('ispl', 2, 0x3e),
        Register('usp', 4, 0x40),
        Register('usph', 2, 0x40),
        Register('uspl', 2, 0x42),
        Register('intbase', 4, 0x44),
        Register('intbaseh', 2, 0x44),
        Register('intbasel', 2, 0x46),
        Register('psr', 2, 0x50),
        Register('cfg', 2, 0x64),
        Register('dbs', 2, 0x6e),
        Register('dsr', 2, 0x70),
        Register('dcr', 4, 0x72),
        Register('dcrh', 2, 0x72),
        Register('dcrl', 2, 0x74),
        Register('car0', 4, 0x76),
        Register('car0h', 2, 0x76),
        Register('car0l', 2, 0x78),
        Register('car1', 4, 0x7a),
        Register('car1h', 2, 0x7a),
        Register('car1l', 2, 0x7c),
        Register('dbs_1', 4, 0x82),
        Register('dsr_1', 4, 0x86),
        Register('cfg_1', 4, 0x8a),
        Register('psr_1', 4, 0x8e)
    ]

register_arch(['cr16c:le:16:default'], 16, Endness.LE, ArchPcode_CR16C_LE_16_default)



###
### This file was automatically generated
###

from archinfo.arch import register_arch, Endness, Register

from .common import ArchPcode


class ArchPcode_6809_BE_16_default(ArchPcode):
    name = '6809:BE:16:default'
    pcode_arch = '6809:BE:16:default'
    description = '6809 Microprocessor'
    bits = 16
    ip_offset = 0x10
    sp_offset = 0x18
    bp_offset = sp_offset
    instruction_endness = Endness.BE
    register_list = [
        Register('d', 2, 0x0),
        Register('a', 1, 0x0),
        Register('b', 1, 0x1),
        Register('cc', 1, 0x8),
        Register('dp', 1, 0x9),
        Register('pc', 2, 0x10, alias_names=('ip',)),
        Register('x', 2, 0x12),
        Register('y', 2, 0x14),
        Register('u', 2, 0x16),
        Register('s', 2, 0x18),
        Register('exg16_r0', 2, 0x20),
        Register('exg8h_r0', 1, 0x20),
        Register('exg8l_r0', 1, 0x21),
        Register('exg16_r1', 2, 0x22),
        Register('exg8h_r1', 1, 0x22),
        Register('exg8l_r1', 1, 0x23)
    ]

register_arch(['6809:be:16:default'], 16, Endness.BE, ArchPcode_6809_BE_16_default)



# pylint:disable=import-outside-toplevel
import os.path
import sys
import importlib.util
from importlib.machinery import ModuleSpec


class FastPkgResources:
    """
    A class that replaces pkg_resources. It only implements resource_filename() and forwards all unimplemented
    attributes to the original pkg_resources.
    """

    def __getattribute__(self, name):
        try:
            return object.__getattribute__(self, name)
        except AttributeError:
            # fallback to the real pkg_resources
            remove_fake_pkg_resources()
            import pkg_resources
            return getattr(pkg_resources, name)

    @staticmethod
    def __spec__():
        return ModuleSpec("pkg_resources", None)

    def resource_filename(self, package, resource_name):
        r = self._resource_filename(package, resource_name)
        if r is None:
            # fallback to the real pkg_resources
            remove_fake_pkg_resources()
            import pkg_resources
            return pkg_resources.resource_filename(package, resource_name)
        return r

    @staticmethod
    def _resource_filename(package, resource_name):
        spec = importlib.util.find_spec(package)
        if spec is None:
            return None
        if not spec.origin:
            return None
        if os.path.isfile(spec.origin):
            # get the directory
            base_dir = os.path.dirname(spec.origin)
        else:
            base_dir = spec.origin
        resource_path = os.path.join(base_dir, resource_name)
        if os.path.exists(resource_path):
            return resource_path
        return None


def import_fake_pkg_resources(force=False):
    if force or 'pkg_resources' not in sys.modules:
        sys.modules['pkg_resources'] = FastPkgResources()


def remove_fake_pkg_resources():
    if isinstance(sys.modules.get('pkg_resources', None), FastPkgResources):
        sys.modules.pop('pkg_resources')



###
### This file was automatically generated
###

from archinfo.arch import register_arch, Endness, Register

from .common import ArchPcode


class ArchPcode_6502_LE_16_default(ArchPcode):
    name = '6502:LE:16:default'
    pcode_arch = '6502:LE:16:default'
    description = '6502 Microcontroller Family'
    bits = 16
    ip_offset = 0x20
    sp_offset = 0x22
    bp_offset = sp_offset
    instruction_endness = Endness.LE
    register_list = [
        Register('a', 1, 0x0),
        Register('x', 1, 0x1),
        Register('y', 1, 0x2),
        Register('p', 1, 0x3),
        Register('pc', 2, 0x20, alias_names=('ip',)),
        Register('pcl', 1, 0x20),
        Register('pch', 1, 0x21),
        Register('sp', 2, 0x22),
        Register('s', 1, 0x22),
        Register('sh', 1, 0x23),
        Register('n', 1, 0x30),
        Register('v', 1, 0x31),
        Register('b', 1, 0x32),
        Register('d', 1, 0x33),
        Register('i', 1, 0x34),
        Register('z', 1, 0x35),
        Register('c', 1, 0x36)
    ]

register_arch(['6502:le:16:default'], 16, Endness.LE, ArchPcode_6502_LE_16_default)



import angr

class stat(angr.SimProcedure):

    def run(self, file_path, stat_buf):
        # this is a dummy for now
        stat = self.state.posix.fstat(0)
        # TODO: make arch-neutral
        self._store_amd64(stat_buf, stat)
        return 0

    def _store_amd64(self, stat_buf, stat):
        store = lambda offset, val: self.state.memory.store(stat_buf + offset, val)

        store(0x00, stat.st_dev)
        store(0x08, stat.st_ino)
        store(0x10, stat.st_nlink)
        store(0x18, stat.st_mode)
        store(0x1c, stat.st_uid)
        store(0x20, stat.st_gid)
        store(0x24, self.state.solver.BVV(0, 32))
        store(0x28, stat.st_rdev)
        store(0x30, stat.st_size)
        store(0x38, stat.st_blksize)
        store(0x40, stat.st_blocks)
        store(0x48, stat.st_atime)
        store(0x50, stat.st_atimensec)
        store(0x58, stat.st_mtime)
        store(0x60, stat.st_mtimensec)
        store(0x68, stat.st_ctime)
        store(0x70, stat.st_ctimensec)
        store(0x78, self.state.solver.BVV(0, 64))
        store(0x80, self.state.solver.BVV(0, 64))
        store(0x88, self.state.solver.BVV(0, 64))
